#ifndef __XMLIO_NC4_DATA_OUTPUT__
#define __XMLIO_NC4_DATA_OUTPUT__

/// xmlioserver headers ///
#include "xmlioserver_spl.hpp"
#include "onetcdf4.hpp"
#include "data_output.hpp"

namespace xmlioserver
{
   namespace io
   {
      /// ////////////////////// Déclarations ////////////////////// ///

      class CNc4DataOutput
         : protected CONetCDF4
         , public virtual CDataOutput
      {
         public :

            /// Définition de type ///
            typedef CONetCDF4   SuperClassWriter;
            typedef CDataOutput SuperClass;

            /// Constructeurs ///
            CNc4DataOutput
               (const StdString & filename, bool exist);
            CNc4DataOutput
               (const StdString & filename, bool exist, bool multigroup, MPI_Comm comm_server);
            CNc4DataOutput
               (const StdString & filename, bool exist, bool multigroup, comm::MPIComm comm_server, bool);

            CNc4DataOutput(const CNc4DataOutput & dataoutput);       // Not implemented.
            CNc4DataOutput(const CNc4DataOutput * const dataoutput); // Not implemented.

            /// Accesseur ///
            const StdString & getFileName(void) const;

            /// Destructeur ///
            virtual ~CNc4DataOutput(void);

         protected :

            /// Ecriture ///
            virtual void writeDomain_(const boost::shared_ptr<tree::CDomain> domain);
            virtual void writeAxis_  (const boost::shared_ptr<tree::CAxis>   axis);
            virtual void writeField_ (const boost::shared_ptr<tree::CField>  field);
            virtual void writeFile_  (const boost::shared_ptr<tree::CFile>   file);


         protected :
         
            void writeLocalAttributes(int ibegin, int iend, int jbegin, int jend, StdString domid);

            void writeFileAttributes(const StdString & name,
                                     const StdString & description,
                                     const StdString & conventions,
                                     const StdString & production,
                                     const StdString & timeStamp);

            void writeMaskAttributes(const StdString & mask_name,
                                     int data_dim,
                                     int data_ni     = 0,
                                     int data_nj     = 0,
                                     int data_ibegin = 0,
                                     int data_jbegin = 0);

            void writeAxisAttributes(const StdString & axis_name,
                                     const StdString & axis,
                                     const StdString & standard_name,
                                     const StdString & long_name,
                                     const StdString & units,
                                     const StdString & nav_model);
            template <class T>
               void writeAxisData(const StdString & axis_name,
                                  const ARRAY(T, 1) data,
                                  bool collective, StdSize record,
                                  const std::vector<StdSize> * start = NULL,
                                  const std::vector<StdSize> * count = NULL);

         private :

            /// Traitement ///
            StdString getTimeStamp(void) const;

            /// Propriétés privées ///
            MPI_Comm comm_server;
            const StdString filename;

      }; // class CNc4DataOutput

   } // namespace io
} // namespace xmlioserver

#endif //__XMLIO_NC4_DATA_OUTPUT__
